﻿#pragma once
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                                                      //
//                                                 Vernox数据库C++接口                                                                   //
//                                                                                                                                      //
//      本文件为访问内存数据库的C++接口。Vernox数据库本身还提供COM接口、ODBC和JDBC驱动接口，请根据需要选择接口类型。提供数据库链接虚类和结果集虚  //
//类，用于操作内存数据库中定义的各种接口。由于虚类交互都是指针，使用起来并不方便，模拟COM+的方式在虚类后面添加Ptr的类为智能指针封装类，实现资源的自 //
//动释放管理功能。                                                                                                                       //
//                                                                                                                                      //
//                                                                                                                         技术预研部   //
//                                                                                                                         2016年       //
//                                                                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef MF_INTERFACE_CLIENT_LOAD_FAILED
#define MF_INTERFACE_CLIENT_LOAD_FAILED                   911     //加载VernoxClient.dll库失败
#endif

#ifndef MF_INTERFACE_CLIENT_FUNC_FAILED
#define MF_INTERFACE_CLIENT_FUNC_FAILED                   912     //定位VernoxClient.dll库函数失败
#endif

#ifndef MF_NULL
#define MF_NULL											 -1       //正确，空数据返回方式
#endif

#ifndef MF_OK
#define MF_OK											 0        //正确，无错误
#endif

#ifndef MF_FAILED
#define MF_FAILED                                        1        //失败，通用失败方式（不建议使用） 
#endif

#ifdef WIN32
#ifndef BSTR
typedef wchar_t *   	BSTR;
#endif
#else
#include <dlfcn.h>
#endif

//索贝内存数据库结果集接口
class IVernoxDBRecordset
{
protected:
	//函数名：析构函数
	//描  述：
	//参  数：无
	//返回值：无
	virtual ~IVernoxDBRecordset() {};
public:
	//释放结果集资源函数
	//参数说明：无
	//返回值：	无
	virtual void Release()	= 0;

	//获取结果集行数
	//参数说明：无
	//返回值：	行数
	virtual int GetRowNum()	= 0;

	//获取结果集列数
	//参数说明：无
	//返回值：	列数
	virtual int GetFieldNum() = 0;

	//获取结果集字段名
	//参数说明：nIndex	为列序号
	//返回值：	字段标题指针，无需释放
	virtual wchar_t* FieldTitleW(int nIndex) = 0;

	//获取结果集字段名
	//参数说明：nIndex	为列序号
	//返回值：	字段标题指针，无需释放
	virtual char* FieldTitle(int nIndex) = 0;

	//获取结果集字段名
	//参数说明：nIndex	为列序号
	//返回值：	字段名指针，无需释放
	virtual wchar_t* FieldNameW(int nIndex) = 0;

	//获取结果集字段名
	//参数说明：nIndex	为列序号
	//返回值：	字段名指针，无需释放
	virtual char* FieldName(int nIndex) = 0;

	//获取结果集字段基础表名
	//参数说明：nIndex	为列序号
	//返回值：	字段名指针，无需释放
	virtual char* FieldBaseTableName(int nIndex) = 0;

	//获取结果集字段基础表名
	//参数说明：nIndex	为列序号
	//返回值：	字段名指针，无需释放
	virtual wchar_t* FieldBaseTableNameW(int nIndex) = 0;

	//获取结果集字段名对应序号
	//参数说明：lpFieldName为列标题
	//返回值：	字段序号，失败返回-1
	virtual int FieldIndex(const wchar_t* lpFieldName) = 0;

	//获取结果集字段名对应序号
	//参数说明：lpFieldName为列标题(UTF8)
	//返回值：	字段序号，失败返回-1
	virtual int FieldIndex(const char* lpFieldName) = 0;

	//获取结果集字段类型
	//参数说明：nIndex	为列序号
	//返回值：	字段类型，0表示失败
	virtual unsigned char FieldType(int nIndex) = 0;

	//获取结果集字段是否主键
	//参数说明：nIndex	为列序号
	//返回值：	1：表示主键列，0表示非主键列
	virtual unsigned char FieldPrimaryKey(int nIndex) = 0;

	//获取结果集字段是否允许为空
	//参数说明：nIndex	为列序号
	//返回值：	1：表示可以为空，0表示不能为空
	virtual unsigned char FieldAllowNull(int nIndex) = 0;

	//获取结果集字段最大允许长度
	//参数说明：nIndex	为列序号
	//返回值：	以字节为单位
	virtual int FieldMaxLen(int nIndex) = 0;

	//克隆一个结果集数据出来
	//参数说明：pNewRs 为克隆的返回结果集
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int Clone(IVernoxDBRecordset * &pNewRs) = 0;

	//向结果集中添加一行
	//参数说明：nDataId返回底层生成的数据ID
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int AddNew(long long &nDataId) = 0;

	//删除结果集中的当前行
	//参数说明：无
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int Delete() = 0;

	//判断结果集中当前行是否第一行
	//参数说明：无
	//返回值：	TRUE表示是第一行，FALSE表示不是第一行
	virtual int bof() = 0;

	//判断结果集中当前行是否最后一行后一行
	//参数说明：无
	//返回值：	TRUE表示是最后一行后一行，FALSE表示不是最后一行后一行
	virtual int eof() = 0;

	//提取一行数据
	//参数说明：无
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int Fetch() = 0;

	//设置结果集中的第一行为当前行
	//参数说明：无
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int MoveFirst() = 0;

	//设置结果集中的下一行为当前行
	//参数说明：无
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int MoveNext() = 0;

	//设置结果集中的前一行为当前行
	//参数说明：无
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int MovePrevious() = 0;

	//设置结果集中的最后一行为当前行
	//参数说明：无
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int MoveLast() = 0;

	//设置给定的DataID行为当前行
	//参数说明：nDataID为数据ID号
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetCurrentRow(long long nDataID) = 0;

	//设置给定的nRowNo行为当前行
	//参数说明：nRowNo为行号
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetCurrentRow(int nRowNo) = 0;

	//获取结果集中当前行的DataID
	//参数说明：nDataID	为返回当前行的DataID
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int GetDataID(long long &llDataID) = 0;

	//获取结果集中当前行指定列的值(32位整数)
	//参数说明：nIndex	为列序号
	//			nValue	为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(int nIndex, int &nValue) = 0;

	//获取结果集中当前行指定列的值(32位整数)
	//参数说明：lpFieldName	为列标题
	//			nValue		为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const wchar_t* lpFieldName, int &nValue) = 0;

	//获取结果集中当前行指定列的值(32位整数)
	//参数说明：lpFieldName	为列序号
	//			nValue		为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const char* lpFieldName, int &nValue) = 0;

	//获取结果集中当前行指定列的值(64位整数)
	//参数说明：nIndex	为列序号
	//			nValue	为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(int nIndex, __int64 &nValue) = 0;

	//获取结果集中当前行指定列的值(64位整数)
	//参数说明：lpFieldName	为列标题
	//			nValue		为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const wchar_t* lpFieldName, __int64 &nValue) = 0;

	//获取结果集中当前行指定列的值(64位整数)
	//参数说明：lpFieldName	为列标题
	//			nValue		为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const char* lpFieldName, __int64 &nValue) = 0;

	//获取结果集中当前行指定列的值(日期型数)
	//参数说明：nIndex	为列序号
	//			stValue	为返回指定列的值。tm时间日期结构体说明：
	//					tm_year为年份，其值等于实际年份减去1900
	//					tm_mon为月份（从一月开始，0代表一月） - 取值区间为[0,11]
	//					tm_mday为一个月中的日期 - 取值区间为[1,31]
	//					tm_hour为时 - 取值区间为[0,23]
	//					tm_min为分 - 取值区间为[0,59]
	//					tm_sec为秒 – 取值区间为[0,59]
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(int nIndex, tm &stValue) = 0;

	//获取结果集中当前行指定列的值(日期型数)
	//参数说明：lpFieldName	为列标题
	//			stValue	为返回指定列的值。tm时间日期结构体说明：
	//					tm_year为年份，其值等于实际年份减去1900
	//					tm_mon为月份（从一月开始，0代表一月） - 取值区间为[0,11]
	//					tm_mday为一个月中的日期 - 取值区间为[1,31]
	//					tm_hour为时 - 取值区间为[0,23]
	//					tm_min为分 - 取值区间为[0,59]
	//					tm_sec为秒 – 取值区间为[0,59]
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const wchar_t* lpFieldName, tm &stValue) = 0;

	//获取结果集中当前行指定列的值(日期型数)
	//参数说明：lpFieldName	为列标题
	//			stValue	为返回指定列的值。tm时间日期结构体说明：
	//					tm_year为年份，其值等于实际年份减去1900
	//					tm_mon为月份（从一月开始，0代表一月） - 取值区间为[0,11]
	//					tm_mday为一个月中的日期 - 取值区间为[1,31]
	//					tm_hour为时 - 取值区间为[0,23]
	//					tm_min为分 - 取值区间为[0,59]
	//					tm_sec为秒 – 取值区间为[0,59]
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const char* lpFieldName, tm &stValue) = 0;

	//获取结果集中当前行指定列的值(浮点数)
	//参数说明：nIndex	为列序号
	//			dblValue为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(int nIndex, double &dblValue) = 0;

	//获取结果集中当前行指定列的值(浮点数)
	//参数说明：lpFieldName	为列标题
	//			dblValue	为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const wchar_t* lpFieldName, double &dblValue) = 0;

	//获取结果集中当前行指定列的值(浮点数)
	//参数说明：lpFieldName	为列标题
	//			dblValue	为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const char* lpFieldName, double &dblValue) = 0;

	//获取结果集中当前行指定列的值(字符串数,UTF8编码格式)
	//参数说明：nIndex	为列序号
	//			pValue	为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(int nIndex, char* &pValue) = 0;

	//获取结果集中当前行指定列的值(字符串数,UTF8编码格式)
	//参数说明：lpFieldName	为列标题
	//			pValue		为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const wchar_t* lpFieldName, char* &pValue) = 0;

	//获取结果集中当前行指定列的值(字符串数,UTF8编码格式)
	//参数说明：lpFieldName	为列标题
	//			pValue		为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const char* lpFieldName, char* &pValue) = 0;

	//获取结果集中当前行指定列的值(字符串数,UNICODE编码格式)
	//参数说明：nIndex	为列序号
	//			pValue	为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(int nIndex, wchar_t* &pValue) = 0;

	//获取结果集中当前行指定列的值(字符串数,UNICODE编码格式)
	//参数说明：lpFieldName	为列标题
	//			pValue		为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const wchar_t* lpFieldName, wchar_t* &pValue) = 0;

	//获取结果集中当前行指定列的值(字符串数,UNICODE编码格式)
	//参数说明：lpFieldName	为列标题
	//			pValue		为返回指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const char* lpFieldName, wchar_t* &pValue) = 0;

	//获取结果集中当前行指定列的值(二进制数)
	//参数说明：nIndex	为列序号
	//			lpValue	为返回指定列的值数据指针
	//			nSize	为返回指定列的值数据大小
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(int nIndex, unsigned char * &lpValue, int &nSize) = 0;

	//获取结果集中当前行指定列的值(二进制数)
	//参数说明：lpFieldName为列标题
	//			lpValue	为返回指定列的值数据指针
	//			nSize	为返回指定列的值数据大小
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const wchar_t* lpFieldName, unsigned char * &lpValue, int &nSize) = 0;

	//获取结果集中当前行指定列的值(二进制数)
	//参数说明：lpFieldName为列标题
	//			lpValue	为返回指定列的值数据指针
	//			nSize	为返回指定列的值数据大小
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int FieldValue(const char* lpFieldName, unsigned char * &lpValue, int &nSize) = 0;

	//设置结果集中当前行指定列为空值
	//参数说明：nIndex	为列序号
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetFieldNull(int nIndex) = 0;

	//设置结果集中当前行指定列为空值
	//参数说明：lpFieldName	为列标题
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetFieldNull(const wchar_t* lpFieldName) = 0;

	//设置结果集中当前行指定列为空值
	//参数说明：lpFieldName	为列标题
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetFieldNull(const char* lpFieldName) = 0;

	//设置结果集中当前行指定列的值(32位整数)
	//参数说明：nIndex	为列序号
	//			nValue	为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(int nIndex, int nValue) = 0;

	//设置结果集中当前行指定列的值(32位整数)
	//参数说明：lpFieldName	为列标题
	//			nValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const wchar_t* lpFieldName, int nValue) = 0;

	//设置结果集中当前行指定列的值(32位整数)
	//参数说明：lpFieldName	为列标题
	//			nValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const char* lpFieldName, int nValue) = 0;

	//设置结果集中当前行指定列的值(64位整数)
	//参数说明：nIndex	为列序号
	//			nValue	为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(int nIndex, __int64 nValue) = 0;

	//设置结果集中当前行指定列的值(64位整数)
	//参数说明：lpFieldName	为列标题
	//			nValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const wchar_t* lpFieldName, __int64 nValue) = 0;

	//设置结果集中当前行指定列的值(64位整数)
	//参数说明：lpFieldName	为列标题
	//			nValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const char* lpFieldName, __int64 nValue) = 0;

	//设置结果集中当前行指定列的值(日期型数)
	//参数说明：nIndex	为列序号
	//			stValue	为设置指定列的值。tm时间日期结构体说明：
	//					tm_year为年份，其值等于实际年份减去1900
	//					tm_mon为月份（从一月开始，0代表一月） - 取值区间为[0,11]
	//					tm_mday为一个月中的日期 - 取值区间为[1,31]
	//					tm_hour为时 - 取值区间为[0,23]
	//					tm_min为分 - 取值区间为[0,59]
	//					tm_sec为秒 – 取值区间为[0,59]
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(int nIndex, const tm &stValue) = 0;

	//设置结果集中当前行指定列的值(日期型数)
	//参数说明：lpFieldName	为列标题
	//			stValue	为设置指定列的值。tm时间日期结构体说明：
	//					tm_year为年份，其值等于实际年份减去1900
	//					tm_mon为月份（从一月开始，0代表一月） - 取值区间为[0,11]
	//					tm_mday为一个月中的日期 - 取值区间为[1,31]
	//					tm_hour为时 - 取值区间为[0,23]
	//					tm_min为分 - 取值区间为[0,59]
	//					tm_sec为秒 – 取值区间为[0,59]
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const wchar_t* lpFieldName, const tm &stValue) = 0;

	//设置结果集中当前行指定列的值(日期型数)
	//参数说明：lpFieldName	为列标题
	//			stValue	为设置指定列的值。tm时间日期结构体说明：
	//					tm_year为年份，其值等于实际年份减去1900
	//					tm_mon为月份（从一月开始，0代表一月） - 取值区间为[0,11]
	//					tm_mday为一个月中的日期 - 取值区间为[1,31]
	//					tm_hour为时 - 取值区间为[0,23]
	//					tm_min为分 - 取值区间为[0,59]
	//					tm_sec为秒 – 取值区间为[0,59]
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const char* lpFieldName, const tm &stValue) = 0;

	//设置结果集中当前行指定列的值(浮点数)
	//参数说明：nIndex		为列序号
	//			dblValue	为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(int nIndex, double dblValue) = 0;

	//设置结果集中当前行指定列的值(浮点数)
	//参数说明：lpFieldName	为列标题
	//			dblValue	为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const wchar_t* lpFieldName, double dblValue) = 0;

	//设置结果集中当前行指定列的值(浮点数)
	//参数说明：lpFieldName	为列标题
	//			dblValue	为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const char* lpFieldName, double dblValue) = 0;

	//设置结果集中当前行指定列的值(UTF8编码格式)
	//参数说明：nIndex		为列序号
	//			lpValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(int nIndex, const char* lpValue) = 0;

	//设置结果集中当前行指定列的值(UTF8编码格式)
	//参数说明：lpFieldName	为列标题
	//			lpValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const wchar_t* lpFieldName, const char* lpValue) = 0;

	//设置结果集中当前行指定列的值(UTF8编码格式)
	//参数说明：lpFieldName	为列标题
	//			lpValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const char* lpFieldName, const char* lpValue) = 0;

	//设置结果集中当前行指定列的值(Unicode编码格式)
	//参数说明：nIndex		为列序号
	//			lpValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(int nIndex, const wchar_t* lpValue) = 0;

	//设置结果集中当前行指定列的值(Unicode编码格式)
	//参数说明：lpFieldName为列标题
	//			lpValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const wchar_t* lpFieldName, const wchar_t* lpValue) = 0;

	//设置结果集中当前行指定列的值(Unicode编码格式)
	//参数说明：lpFieldName	为列标题
	//			lpValue		为设置指定列的值
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const char* lpFieldName, const wchar_t* lpValue) = 0;

	//设置结果集中当前行指定列的值(二进制数)
	//参数说明：nIndex		为列序号
	//			lpValue		为设置指定列的值数据指针
	//			nSize		为设置指定列的值数据大小
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(int nIndex, const unsigned char * lpValue, int nSize) = 0;

	//设置结果集中当前行指定列的值(二进制数)
	//参数说明：lpFieldName	为列标题
	//			lpValue		为设置指定列的值数据指针
	//			nSize		为设置指定列的值数据大小
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const wchar_t* lpFieldName, const unsigned char * lpValue, int nSize) = 0;

	//设置结果集中当前行指定列的值(二进制数)
	//参数说明：lpFieldName	为列标题
	//			lpValue		为设置指定列的值数据指针
	//			nSize		为设置指定列的值数据大小
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const char* lpFieldName, const unsigned char * lpValue, int nSize) = 0;

	//获取结果集字段类别
	//参数说明：nIndex	为列序号
	//返回值：	字段类型，0表示失败，1：为原始字段
	virtual unsigned char FieldCategory(int nIndex) = 0;

	//将结果集另存为CSV文件
	virtual int RecordsetToCSV(char* &lpCSVValue) = 0;

#ifdef WIN32
	//设置结果集中当前行指定列的值(可变类型)
	//参数说明：nIndex		为列序号
	//			pVarSrc		为设置指定列的值，为VARIANT *类型
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(int nIndex, const void * pVarSrc, int nSize) = 0;

	//设置结果集中当前行指定列的值(可变类型)
	//参数说明：lpFieldName	为列标题
	//			pVarSrc		为设置指定列的值，为VARIANT *类型
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const wchar_t* lpFieldName, const void * pVarSrc, int nSize) = 0;

	//设置结果集中当前行指定列的值(可变类型)
	//参数说明：lpFieldName	为列标题
	//			pVarSrc		为设置指定列的值，为VARIANT *类型
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const char* lpFieldName, const void * pVarSrc, int nSize) = 0;

	//设置结果集中当前行指定列的值(BSTR类型)
	//参数说明：nIndex		为列序号
	//			bstrValue	为设置指定列的值
	//			nLen		为设置指定列的值长度
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(int nIndex, const BSTR& bstrValue, int nLen) = 0;

	//设置结果集中当前行指定列的值(BSTR类型)
	//参数说明：lpFieldName	为列标题
	//			bstrValue	为设置指定列的值
	//			nLen		为设置指定列的值长度
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const wchar_t* lpFieldName, const BSTR& bstrValue, int nLen) = 0;

	//设置结果集中当前行指定列的值(BSTR类型)
	//参数说明：lpFieldName	为列标题
	//			bstrValue	为设置指定列的值
	//			nLen		为设置指定列的值长度
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int SetField(const char* lpFieldName, const BSTR& bstrValue, int nLen) = 0;
#endif
};


//索贝内存数据库连接接口
class IVernoxDBConnection
{
protected:
	//函数名：析构函数
	//描  述：
	//参  数：无
	//返回值：无
	virtual ~IVernoxDBConnection() {};

public:
	//释放数据库连接对象资源函数
	//参数说明：无
	//返回值：	无
	virtual void Release()	= 0;

	//连接到数据库函数
	//参数说明：lpszServerName	为服务名连接串，在Vernox.ini文件中定义。
	//			lpszUserName	为登录数据库用户名
	//			lpszPassword	为登录数据库用户的口令
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int Open(char * lpszServerName, char * lpszUserName, char * lpszPassword)		= 0;

	//连接到数据库函数
	//参数说明：lpszServer		为IP地址或者服务名连接串，在Vernox.ini文件中定义。
	//			usPort			为服务监听端口，当lpszServer配置为IP时有效
	//			lpszUserName	为登录数据库用户名
	//			lpszPassword	为登录数据库用户的口令
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int Open(char * lpszServer, unsigned short usPort, char * lpszUserName, char * lpszPassword)		= 0;

	//关闭数据库连接函数
	//参数说明：无
	//返回值：	无
	virtual void Close()	= 0;

	//数据库SQL解析函数，分配一个游标句柄
	//参数说明：lpszSqlText		为解析的SQL语句
	//			hCursor			为解析游标句柄
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int Parse(char * lpszSqlText, void * &hCursor)	= 0;

	//数据库SQL变量绑定函数
	//参数说明：hCursor			为绑定数据的游标句柄
	//			nIndex			为绑定变量编号，从1开始
	//			bDataType		为绑定数据类型
	//			lpData			为绑定数据指针
	//			nDataSize		为绑定数据大小，常规类型使用sizeof计算数据大小，字符串为字符串长度（包含字符串结束符0），BLOB为二进制大小。
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int Bind(void * hCursor, int nIndex, unsigned char bDataType, void * lpData, int nDataSize)	= 0;

	//数据库执行函数，自动释放游标内存，不能在调CloseCursor手动释放
	//参数说明：hCursor			为执行的游标句柄
	//			lAffectCount	为执行影响条数
	//			lpDescription	为返回执行描述信息，默认情况为空(NULL)
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int Execute(void * hCursor, int &lAffectCount, void * lpDescription = NULL)	= 0;

	//手动释放游标内存
	//参数说明：hCursor			为游标句柄
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int CloseCursor(void * hCursor)	= 0;

	//数据库直接执行SQL函数
	//参数说明：lpszSqlText		为执行的SQL语句
	//			lAffectCount	为执行SQL语句影响条数
	//			lpDescription	为返回SQL语句的执行描述信息，默认情况为空(NULL)
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int Execute(char * lpszSqlText, int &lAffectCount, void * lpDescription = NULL)	= 0;

	//数据库查询数据到结果集函数
	//参数说明：lpszSqlText		为执行的SQL语句
	//			lAffectCount	为执行SQL语句影响条数
	//			lpDescription	为返回SQL语句的执行描述信息，默认情况为空(NULL)
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int GetRecordset(char * lpszSqlText, IVernoxDBRecordset** pRecordset, void * lpDescription = NULL)	= 0;

	//结果集更新执行函数
	//参数说明：lpszSqlText		为执行的SQL语句
	//			lAffectCount	为执行SQL语句影响条数
	//			lpDescription	为返回SQL语句的执行描述信息，默认情况为空(NULL)
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int UpdateRecordset(IVernoxDBRecordset* pRecordset, void * lpDescription = NULL)	= 0;

	//开启一个事务逻辑（非数据库锁数据事务），在整个数据库范围内以事务数据作为唯一标识，有效期最长1分钟
	//参数说明：lpTransactionData	为事务数据
	//			nTransactionID		为事务ID
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int StartTransactionLogic(char* lpTransactionData, int& nTransactionID)	= 0;

	//结束一个事务逻辑（非数据库锁数据事务）
	//参数说明：nTransactionID		为事务ID
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int StopTransactionLogic(int nTransactionID) = 0;

	//获取数据库执行计划函数
	//参数说明：lpszSqlText		为执行的SQL语句
	//			lpPlanInfo		为执行SQL语句的执行计划
	//			lpDescription	为返回SQL语句的执行描述信息，默认情况为空(NULL)
	//返回值：	0表示正常执行结束，其他表示错误码
	//virtual int ExecutePlanInfo(LPCTSTR lpszSqlText, void * lpPlanInfo, void * lpDescription = NULL)	= 0;

	//结果集更新执行函数
	//参数说明：lpBuffer		为恢复的重做日志
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int VernoxRedo(unsigned char * lpBuffer, unsigned short usDatabaseGuid)	= 0;

	//设置当前用户连接库名
	//参数说明：lpszDBName	为库名，默认登录到Vernox库
	//返回值：	0表示正常执行结束，其他表示错误码
	virtual int UseDatabase(const char * lpszDBName)	= 0;
};

//创建Vernox数据库链接实例函数
//参数说明：pConnection		为返回创建好的数据库链接指针，需要调
//			lAffectCount	为执行SQL语句影响条数
//			lpDescription	为返回SQL语句的执行描述信息，默认情况为空(NULL)
//返回值：	0表示正常执行结束，其他表示错误码
//注意事项，VERNOX定义后表示直接使用Vernox的Lib方式访问，否则为使用动态VernoxClient.dll(libVernoxInterface.so)实现。
//			WIN32表示系统平台为Windows（包括32位和64位），否则表示Linux系统；Q_OS_WIN表示使用Windows下的QT开发工具。
#ifndef VERNOX
#ifdef WIN32
inline int CreateVernoxDBInstance(IVernoxDBConnection * &pConnection)
{
#ifndef Q_OS_WIN
	static HMODULE hInstance = NULL;
	typedef int (*CreateVernoxDBInstancePtr)(IVernoxDBConnection * &pConnection);
	CreateVernoxDBInstancePtr pInstancePtr = NULL;

	if(hInstance == NULL)
	{
		int nLastError;
		TCHAR tszPath[MAX_PATH];
		CString	strDir,strPath,strFile, strText;

		memset(tszPath, 0, sizeof(tszPath));
		GetModuleFileName(NULL, tszPath, MAX_PATH);
		strDir = tszPath;
		strPath = strDir.Left(strDir.ReverseFind(_T('\\')))	;
		GetCurrentDirectory(MAX_PATH, tszPath);
		strDir  = tszPath;
#if _DEBUG
		strFile = _T("VernoxClientD.dll");
#else
		strFile = _T("VernoxClient.dll");
#endif
		SetCurrentDirectory(strPath);
		hInstance = ::LoadLibrary(strFile);
		nLastError = GetLastError();
		SetCurrentDirectory(strDir);
		if(hInstance == NULL)
		{
			return MF_INTERFACE_CLIENT_LOAD_FAILED;
		}
		pInstancePtr = (CreateVernoxDBInstancePtr)GetProcAddress(hInstance, "CreateVernoxDBInstance");
	}
	else
	{
		pInstancePtr = (CreateVernoxDBInstancePtr)GetProcAddress(hInstance, "CreateVernoxDBInstance");
	}
	if(pInstancePtr == NULL)
	{
		return MF_INTERFACE_CLIENT_FUNC_FAILED;
	}

	return pInstancePtr(pConnection);
#else
	static QLibrary* qlib = nullptr;
	typedef int(*CreateVernoxDBInstancePtr)(IVernoxDBConnection * &pConnection);
	CreateVernoxDBInstancePtr pInstancePtr = NULL;
	if ( nullptr == qlib )
	{
		QString qstrFile = "VernoxClient.dll";
		qlib = new QLibrary(qstrFile);
		if (!qlib->load())
		{
			return MF_INTERFACE_CLIENT_LOAD_FAILED;
		}
	}
	std::string sym = "CreateVernoxDBInstance";
	pInstancePtr = (CreateVernoxDBInstancePtr)qlib->resolve(sym.c_str());
	if (pInstancePtr == NULL)
	{
		return MF_INTERFACE_CLIENT_FUNC_FAILED;
	}

	return pInstancePtr(pConnection);
#endif
}
#else
int VernoxGetModuleFileName(TCHAR* sModuleName, TCHAR* sFileName, int nSize)
{
	int nRet = -1;

	if(sModuleName == NULL)
	{
		readlink("/proc/self/exe", sFileName, nSize);
		return MF_OK;
	}
	if(_tcschr(sModuleName, '/') != NULL)
	{
		_tcscpy(sFileName, sModuleName);
	}
	else
	{
		char* sPath = getenv("PATH");
		char* pHead = sPath;
		char* pTail = NULL;
		while(pHead != NULL && *pHead != '\x0')
		{
			pTail = strchr(pHead,':');
			if(pTail != NULL)
			{
				_tcsncpy( sFileName, pHead, pTail-pHead );
				sFileName[pTail-pHead] = '\x0';
				pHead = pTail+1;
			}
			else
			{
				_tcscpy( sFileName, pHead );
				pHead = NULL;
			}

			int nLen = _tcslen(sFileName);
			if(sFileName[nLen] != '/')
			{
				sFileName[nLen] = '/';
			}
			_tcscpy(sFileName+nLen+1,sModuleName);

			//判断文件是否存在
			if(0 == _taccess(sFileName, F_OK))
			{
				nRet = 0;
				break;
			}
		}
	}
	return nRet;
}

inline int CreateVernoxDBInstance(IVernoxDBConnection * &pConnection)
{
	static void *hInstance = NULL;
	typedef int (*CreateVernoxDBInstancePtr)(IVernoxDBConnection * &pConnection);
	CreateVernoxDBInstancePtr pInstancePtr = NULL;
	
	TCHAR soPath[MAX_PATH];
	char *pathDir = NULL;
	memset(soPath, 0, sizeof(soPath));
	
	VernoxGetModuleFileName(NULL, soPath, MAX_PATH);
	pathDir = strrchr(soPath, '/');
	memcpy(pathDir + 1, "libVernoxInterface.so", strlen("libVernoxInterface.so") + 1);

	if(hInstance == NULL)
	{
		hInstance = dlopen(soPath, RTLD_NOW);

		if(hInstance == NULL)
		{
			return MF_INTERFACE_CLIENT_LOAD_FAILED;
		}
		
		pInstancePtr = (CreateVernoxDBInstancePtr)dlsym(hInstance, "CreateVernoxDBInstance2");
	}
	else
	{
		pInstancePtr = (CreateVernoxDBInstancePtr)dlsym(hInstance, "CreateVernoxDBInstance2");
	}
	
	if(pInstancePtr == NULL)
	{
		return MF_INTERFACE_CLIENT_FUNC_FAILED;
	}

	return pInstancePtr(pConnection);	
}
#endif
#else
extern int CreateVernoxDBInstance(IVernoxDBConnection * &pConnection);
#endif

//数据库结果集智能指针，推荐使用此类
class IVernoxDBRecordsetPtr
{
private:
	//禁止做复制操作
	IVernoxDBRecordsetPtr& operator = (const IVernoxDBRecordsetPtr&);
	operator IVernoxDBRecordsetPtr ();
protected:
	IVernoxDBRecordset * m_pDBRecordset;
public:
	//构造函数
	IVernoxDBRecordsetPtr()
	{
		m_pDBRecordset = NULL;
	}
	//析构函数
	~IVernoxDBRecordsetPtr()
	{
		if(m_pDBRecordset != NULL)
		{
			m_pDBRecordset->Release();
			m_pDBRecordset = NULL;
		}
	}
	//结果集克隆
	int Clone(IVernoxDBRecordsetPtr &pDesRs)
	{
		if(m_pDBRecordset != NULL)
		{
			pDesRs.Release();
			return m_pDBRecordset->Clone(pDesRs.m_pDBRecordset);
		}
		return MF_FAILED;
	}
	//重载指针操作符
	IVernoxDBRecordset* operator->() const
	{
		return m_pDBRecordset;
	}

	IVernoxDBRecordset**	operator& ()
	{
		Release();
		return &m_pDBRecordset;
	}

	operator IVernoxDBRecordset* ()
	{
		return m_pDBRecordset;
	}

	bool operator ==( const IVernoxDBRecordsetPtr & s)const
	{
		return (m_pDBRecordset == s.m_pDBRecordset);
	}

	bool operator ==( const IVernoxDBRecordset* p)const
	{
		return (m_pDBRecordset == p);
	}

	bool operator !=( const IVernoxDBRecordsetPtr & s)const
	{
		return (m_pDBRecordset != s.m_pDBRecordset);
	}

	bool operator !=( const IVernoxDBRecordset* p)const
	{
		return (m_pDBRecordset != p);
	}
	//释放实例
	void Release()
	{
		IVernoxDBRecordset* pTemp = m_pDBRecordset;
		if (pTemp)
		{
			m_pDBRecordset = NULL;
			pTemp->Release();
		}
	}

	// Attach to an existing interface
	void Attach(IVernoxDBRecordset* pDBRecordset)
	{
		Release();
		m_pDBRecordset = pDBRecordset;
	}

	// Detach the interface (does not Release)
	IVernoxDBRecordset* Detach()
	{
		IVernoxDBRecordset* pTemp = m_pDBRecordset;
		m_pDBRecordset = NULL;
		return pTemp;
	}
};

//数据库链接智能指针，推荐使用此类
class IVernoxDBConnectionPtr
{
private:
	//禁止做
	IVernoxDBConnectionPtr& operator = (const IVernoxDBConnectionPtr&);
protected:
	IVernoxDBConnection * m_pDBConnection;
public:
	//构造函数
	IVernoxDBConnectionPtr()
	{
		m_pDBConnection = NULL;
	}
	//析构函数
	~IVernoxDBConnectionPtr()
	{
		if(m_pDBConnection != NULL)
		{
			m_pDBConnection->Release();
			m_pDBConnection = NULL;
		}
	}

	//重载指针操作符
	IVernoxDBConnection* operator->() const
	{
		return m_pDBConnection;
	}

	operator IVernoxDBConnection* ()
	{
		return m_pDBConnection;
	}

	//创建一个实例
	int CreateInstance()
	{
		Release();
		return CreateVernoxDBInstance(m_pDBConnection);
	}

	//释放实例
	void Release()
	{
		IVernoxDBConnection* pTemp = m_pDBConnection;
		if (pTemp)
		{
			m_pDBConnection = NULL;
			pTemp->Release();
		}
	}

	// Attach to an existing interface
	void Attach(IVernoxDBConnection* pDBConnection)
	{
		Release();
		m_pDBConnection = pDBConnection;
	}

	// Detach the interface (does not Release)
	IVernoxDBConnection* Detach()
	{
		IVernoxDBConnection* pTemp = m_pDBConnection;
		m_pDBConnection = NULL;
		return pTemp;
	}
};

